@@ -1,7 +1,8 @@
-# $Id: TranscodeTab.pm,v 1.61.2.1 2002/11/23 13:44:42 joern Exp $
+# $Id: TranscodeTab.pm,v 1.83 2003/02/08 11:29:21 joern Exp $
 
 #-----------------------------------------------------------------------
-# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
+# Copyright (C) 2001-2003 Jrn Reder <joern AT zyn.de>.
+# All Rights Reserved. See file COPYRIGHT for details.
 # 
 # This module is part of Video::DVDRip, which is free software; you can
 # redistribute it and/or modify it under the same terms as Perl itself.
@@ -23,6 +24,7 @@ sub set_transcode_widgets	{ shift->{transcode_widgets}	= $_[1] }
 sub in_transcode_init		{ shift->{in_transcode_init}		}	# href
 sub set_in_transcode_init	{ shift->{in_transcode_init}	= $_[1] }
 
+
 #---------------------------------------------------------------------
 # Build Transcode Tab
 #---------------------------------------------------------------------
@@ -68,24 +70,28 @@ sub create_transcode_tab {
 	my $calculated_storage = $self->create_calculated_storage (
 		hsize_group => $left_hsize_group
 	);
+	my $container_options = $self->create_container_options (
+		hsize_group => $left_hsize_group
+	);
 	
 	# Put frames into table
-	my $table = Gtk::Table->new ( 4, 2, 0 );
+	my $table = Gtk::Table->new ( 5, 2, 0 );
 	$table->show;
 	$table->set_row_spacings ( $TABLE_SPACING );
 	$table->set_col_spacings ( $TABLE_SPACING );
 	$vbox->pack_start ($table, 0, 1, 0);
 
-	$table->attach_defaults ($video_options, 	0, 1, 0, 1);
+	$table->attach_defaults ($container_options, 	0, 1, 0, 1);
+	$table->attach_defaults ($video_options, 	0, 1, 1, 2);
 
-	$table->attach_defaults ($audio_options, 	1, 2, 0, 1);
+	$table->attach_defaults ($audio_options, 	1, 2, 0, 2);
 
-	$table->attach_defaults ($video_bitrate_calc, 	0, 1, 1, 2);
-	$table->attach_defaults ($calculated_storage, 	0, 1, 2, 3);
+	$table->attach_defaults ($video_bitrate_calc, 	0, 1, 2, 3);
+	$table->attach_defaults ($calculated_storage, 	0, 1, 3, 4);
 
-	$table->attach_defaults ($general_options, 	1, 2, 1, 3);
+	$table->attach_defaults ($general_options, 	1, 2, 2, 4);
 
-	$table->attach_defaults ($operate, 		0, 2, 3, 4);
+	$table->attach_defaults ($operate, 		0, 2, 4, 5);
 
 	# connect changed signals
 	my $widgets = $self->transcode_widgets;
@@ -96,18 +102,15 @@ sub create_transcode_tab {
 			       tc_video_framerate tc_execute_afterwards
 			       tc_start_frame tc_end_frame
 			       tc_target_size )) {
-		my $signal = ($attr =~ /^tc_(target_size|video_bitrate|video_codec)$/ )
-			? "changed" : "focus-out-event";
-		$widgets->{$attr}->signal_connect ($signal, sub {
+		$widgets->{$attr}->signal_connect ("changed", sub {
 			my ($widget, $method) = @_;
 			return 1 if not $self->selected_title;
 			return 1 if $self->in_transcode_init;
 			my $title = $self->selected_title;
+
 			$title->$method ( $widget->get_text );
 			if ( $method eq "set_tc_target_size" ) {
-				$title->calc_video_bitrate;
-				$widgets->{tc_video_bitrate}
-					->set_text ( $title->tc_video_bitrate );
+				$self->calc_video_bitrate;
 			}
 			$self->update_storage_labels
 				if $method eq 'set_tc_video_bitrate';
@@ -122,16 +125,20 @@ sub create_transcode_tab {
 					$self->burn_widgets
 					     ->{burn_cd_type_iso}->set_active(1);
 				}
-				if ( $widget->get_text eq 'ffmpeg' ) {
-					$widgets->{tc_video_af6_codec}->set_text ("mpeg4");
-					$title->set_tc_video_af6_codec("mpeg4");
-				} elsif ( $widget->get_text ne 'af6' ) {
-					$widgets->{tc_video_af6_codec}->set_text ("");
-					$title->set_tc_video_af6_codec("");
-				}
 				$self->selected_title->calc_video_bitrate;
+				$self->init_transcode_values ( no_codec_combo => 1 );
+			}
+			if ( $attr eq 'tc_start_frame' or $attr eq 'tc_end_frame' and
+			     $title->tc_video_bitrate_range ) {
+			     	if ($attr eq 'tc_start_frame') {
+					$title->set_tc_start_frame($_[0]->get_text);
+				} else {
+					$title->set_tc_end_frame($_[0]->get_text);
+				}
+				$self->calc_video_bitrate;
 				$self->init_transcode_values;
 			}
+
 			1;
 		}, "set_$attr");
 	}
@@ -146,9 +153,8 @@ sub create_transcode_tab {
 	});
 	
 	# radio button signals
-	
-	foreach my $attr (qw ( tc_use_yuv_internal tc_multipass
-			       tc_preview tc_psu_core )) {
+
+	foreach my $attr (qw ( tc_multipass tc_preview tc_psu_core )) {
 		$widgets->{$attr."_yes"}->signal_connect ( "clicked", sub {
 			my ($widget, $method) = @_;
 			return 1 if not $self->selected_title;
@@ -178,7 +184,7 @@ sub create_transcode_tab {
 	return $vbox;
 }
 
-sub create_video_options {
+sub create_container_options {
 	my $self = shift;
 	my %par = @_;
 	my ($hsize_group) = @par{'hsize_group'};
@@ -186,6 +192,84 @@ sub create_video_options {
 	my ($frame, $frame_hbox, $table, $row, $hbox, $label, $entry);
 	my ($popup_menu, $popup, $item, %popup_entries);
 
+	# Frame
+	$frame = Gtk::Frame->new ("Container options");
+	$frame->show;
+
+	# Frame HBox
+	$frame_hbox = Gtk::HBox->new;
+	$frame_hbox->set_border_width(5);
+	$frame_hbox->show;
+	$frame->add ($frame_hbox);
+
+	# Table
+	$table = Gtk::Table->new ( 1, 2, 0 );
+	$table->show;
+	$table->set_row_spacings ( $TABLE_SPACING );
+	$table->set_col_spacings ( $TABLE_SPACING );
+	$frame_hbox->pack_start ($table, 1, 1, 0);
+
+	# Container Format
+	$row = 0;
+	$hbox = Gtk::HBox->new;
+	$hbox->show;
+	$label = Gtk::Label->new ("Target format");
+	$label->show;
+	$hbox->pack_start($label, 0, 1, 0);
+	$hsize_group->add ($hbox);
+	$table->attach ($hbox, 0, 1, $row, $row+1, 'fill','expand',0,0);
+
+	$popup_menu = Gtk::Menu->new;
+	$popup_menu->show;
+	$popup = Gtk::OptionMenu->new;
+	$popup->show;
+	$popup->set_usize(90,undef);
+	$popup->set_menu($popup_menu);
+
+	foreach my $key ( sort keys %Video::DVDRip::container_formats ) {
+		$item = Gtk::MenuItem->new (
+			$Video::DVDRip::container_formats{$key}
+		);
+		$item->show;
+		$popup_menu->append($item);
+		
+		my $sensitive = 1;
+		$sensitive = 0 if $key eq 'vcd' and not $self->has("mjpegtools");
+		$sensitive = 0 if $key eq 'ogg' and not $self->has("ogmtools");
+		
+		$item->set_sensitive($sensitive);
+
+		$item->signal_connect (
+			"activate", sub {
+				my $title = $self->selected_title;
+				return 1 if not $title;
+				return 1 if $self->in_transcode_init;
+				$self->selected_title
+				     ->set_tc_container($key);
+				$self->init_video_codec_combo;
+				$self->calc_video_bitrate;
+				$self->init_audio_values;
+				$self->init_transcode_values;
+
+				1;
+			}, $key
+		) if $sensitive;
+	}
+	$table->attach ($popup, 1, 2, $row, $row+1, 'fill','expand',0,0);
+
+	$self->transcode_widgets->{tc_container_popup} = $popup;
+
+	return $frame;
+}
+
+sub create_video_options {
+	my $self = shift;
+	my %par = @_;
+	my ($hsize_group) = @par{'hsize_group'};
+
+	my ($frame, $frame_hbox, $table, $row, $hbox, $label, $entry);
+	my ($popup_menu, $popup, $item, %popup_entries, $button);
+
 	# Frame
 	$frame = Gtk::Frame->new ("Video options");
 	$frame->show;
@@ -197,7 +281,7 @@ sub create_video_options {
 	$frame->add ($frame_hbox);
 
 	# Table
-	$table = Gtk::Table->new ( 7, 2, 0 );
+	$table = Gtk::Table->new ( 6, 2, 0 );
 	$table->show;
 	$table->set_row_spacings ( $TABLE_SPACING );
 	$table->set_col_spacings ( $TABLE_SPACING );
@@ -213,18 +297,19 @@ sub create_video_options {
 	$table->attach ($hbox, 0, 1, $row, $row+1, 'fill','expand',0,0);
 	$hsize_group->add ($hbox);
 
-	$hbox = Gtk::HBox->new;
+	$hbox = Gtk::HBox->new (0, 20);
 	$hbox->show;
 
 	$entry = Gtk::Combo->new;
 	$entry->show;
-	$entry->set_popdown_strings ("SVCD","VCD","divx4","divx5","xvid","xvidcvs","ffmpeg","fame","af6");
+#	$entry->set_popdown_strings ("SVCD","VCD","divx4","divx5","xvid","xvidcvs","ffmpeg","fame","af6");
 	$entry->set_usize(80,undef);
 	$hbox->pack_start($entry, 0, 1, 0);
 
 	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
 
 	$self->transcode_widgets->{tc_video_codec} = $entry->entry;
+	$self->transcode_widgets->{tc_video_codec_combo} = $entry;
 
 	# AF6 Codec
 	++$row;
@@ -276,30 +361,6 @@ sub create_video_options {
 
 	$self->transcode_widgets->{tc_video_framerate} = $entry->entry;
 
-	# Use YUV Internal
-	++$row;
-	$hbox = Gtk::HBox->new;
-	$hbox->show;
-	$label = Gtk::Label->new ("YUV internally");
-	$label->show;
-	$hbox->pack_start($label, 0, 1, 0);
-	$table->attach ($hbox, 0, 1, $row, $row+1, 'fill','expand',0,0);
-	$hsize_group->add ($hbox);
-
-	$hbox = Gtk::HBox->new;
-	$hbox->show;
-	my $radio_yes = Gtk::RadioButton->new ("Yes");
-	$radio_yes->show;
-	$hbox->pack_start($radio_yes, 0, 1, 0);
-	my $radio_no = Gtk::RadioButton->new ("No", $radio_yes);
-	$radio_no->show;
-	$hbox->pack_start($radio_no, 0, 1, 0);
-	
-	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
-
-	$self->transcode_widgets->{tc_use_yuv_internal_yes} = $radio_yes;
-	$self->transcode_widgets->{tc_use_yuv_internal_no}  = $radio_no;
-	
 	# Enable DivX Multipass Encoding
 	++$row;
 	$hbox = Gtk::HBox->new;
@@ -338,8 +399,8 @@ sub create_video_options {
 	$popup_menu->show;
 	$popup = Gtk::OptionMenu->new;
 	$popup->show;
-	$popup->set_usize(180,undef);
 	$popup->set_menu($popup_menu);
+	$popup->set_usize (170, undef);
 
 	foreach my $key ( sort keys %Video::DVDRip::deinterlace_filters ) {
 		$item = Gtk::MenuItem->new (
@@ -356,44 +417,37 @@ sub create_video_options {
 			}, $key
 		);
 	}
-	$table->attach ($popup, 1, 2, $row, $row+1, 'fill','expand',0,0);
+	
+	$hbox = Gtk::HBox->new;
+	$hbox->show,
+	$hbox->pack_start ($popup, 0, 1, 0);
+	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
 
 	$self->transcode_widgets->{tc_deinterlace_popup} = $popup;
 
-	# Antialias
+	# Filters
 	++$row;
 	$hbox = Gtk::HBox->new;
 	$hbox->show;
-	$label = Gtk::Label->new ("Antialias mode");
+	$label = Gtk::Label->new ("Filters");
 	$label->show;
 	$hbox->pack_start($label, 0, 1, 0);
 	$table->attach ($hbox, 0, 1, $row, $row+1, 'fill','expand',0,0);
 	$hsize_group->add ($hbox);
 
-	$popup_menu = Gtk::Menu->new;
-	$popup_menu->show;
-	$popup = Gtk::OptionMenu->new;
-	$popup->show;
-	$popup->set_menu($popup_menu);
+	$button = Gtk::Button->new ("  Configure filters & preview...  ");
+	$button->show;
+	$hbox = Gtk::HBox->new;
+	$hbox->show;
+	$hbox->pack_start ($button, 0, 1, 0);
+	$button->signal_connect ( "clicked", sub { $self->transcode_filters } );
 
-	foreach my $key ( sort keys %Video::DVDRip::antialias_filters ) {
-		$item = Gtk::MenuItem->new (
-			$Video::DVDRip::antialias_filters{$key}
-		);
-		$item->show;
-		$popup_menu->append($item);
-		$item->signal_connect (
-			"activate", sub {
-				return 1 if not $self->selected_title;
-				return 1 if $self->in_transcode_init;
-				$self->selected_title
-				     ->set_tc_anti_alias($key)
-			}, $key
-		);
+	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
+
+	if ( $self->version ("transcode") < 603 ) {
+		$button->set_sensitive(0);
+		$button->child->set (" Needs transcode 0.6.3 ");
 	}
-	$table->attach_defaults ($popup, 1, 2, $row, $row+1);
-
-	$self->transcode_widgets->{tc_anti_alias_popup} = $popup;
 
 	return $frame;
 }
@@ -417,7 +471,7 @@ sub create_video_bitrate_calc {
 	$frame->add ($frame_hbox);
 
 	# Table
-	$table = Gtk::Table->new ( 3, 2, 0 );
+	$table = Gtk::Table->new ( 3, 3, 0 );
 	$table->show;
 	$table->set_row_spacings ( $TABLE_SPACING );
 	$table->set_col_spacings ( $TABLE_SPACING );
@@ -440,7 +494,7 @@ sub create_video_bitrate_calc {
 	$popup_menu->show;
 	$popup = Gtk::OptionMenu->new;
 	$popup->show;
-	$popup->set_usize(60,undef);
+	$popup->set_usize (65,undef);
 	$popup->set_menu($popup_menu);
 
 	%popup_entries = (
@@ -460,18 +514,16 @@ sub create_video_bitrate_calc {
 				return 1 if not $title;
 				return 1 if $self->in_transcode_init;
 				$title->set_tc_disc_cnt($key);
-				if ( $title->tc_video_codec ne 'VCD' ) {
-					$title->set_tc_target_size(
-						$key * $title->tc_disc_size,
-					);
-					$self->transcode_widgets
-					     ->{tc_target_size}
-					     ->set_text ($title->tc_target_size);
-				}
+				$title->set_tc_target_size(
+					int($key * $title->tc_disc_size)
+				);
+				$self->transcode_widgets
+				     ->{tc_target_size}
+				     ->set_text ($title->tc_target_size);
 			}, $key
 		);
 	}
-	$hbox->pack_start($popup, 1, 1, 0);
+	$hbox->pack_start($popup, 0, 1, 0);
 	
 	$self->transcode_widgets->{tc_disc_cnt_popup} = $popup;
 
@@ -485,7 +537,7 @@ sub create_video_bitrate_calc {
 		may_empty      => 0,
 	);
 	$entry->show;
-	$entry->set_popdown_strings (650, 700, 760);
+	$entry->set_popdown_strings (650, 703, 800, 870);
 	$entry->set_usize(60,undef);
 	$hbox->pack_start($entry, 0, 1, 0);
 
@@ -494,14 +546,13 @@ sub create_video_bitrate_calc {
 		return 1 if not $title;
 		return 1 if $self->in_transcode_init;
 		$title->set_tc_disc_size ($_[0]->get_text);
-		if ( $title->tc_video_codec ne 'VCD' ) {
-			$title->set_tc_target_size(
-				$title->tc_disc_cnt * $title->tc_disc_size,
-			);
-			$self->transcode_widgets
-			     ->{tc_target_size}
-			     ->set_text ($title->tc_target_size);
-		}
+		$title->set_tc_target_size(
+			int ( $title->tc_disc_cnt *
+			      $title->tc_disc_size ),
+		);
+		$self->transcode_widgets
+		     ->{tc_target_size}
+		     ->set_text ($title->tc_target_size);
 		1;
 	});
 
@@ -511,7 +562,7 @@ sub create_video_bitrate_calc {
 	$label->show;
 	$hbox->pack_start ($label, 0, 1, 0);
 
-	$table->attach ($hbox, 1, 2, $row, $row+1, 'fill','expand',0,0);
+	$table->attach ($hbox, 1, 3, $row, $row+1, 'fill','expand',0,0);
 
 	# Target Size
 	++$row;
@@ -531,7 +582,7 @@ sub create_video_bitrate_calc {
 		may_empty      => 0,
 	);
 	$entry->show;
-	$entry->set_usize(80,undef);
+	$entry->set_usize(65,undef);
 	$hbox->pack_start($entry, 0, 1, 0);
 
 	$label = Gtk::Label->new ("MB");
@@ -541,7 +592,29 @@ sub create_video_bitrate_calc {
 	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
 
 	$self->transcode_widgets->{tc_target_size} = $entry;
+	$self->transcode_widgets->{tc_target_size_label} = $label;
 
+	
+	$hbox = Gtk::HBox->new;
+	$hbox->show;
+	my $checkbox = Gtk::CheckButton->new ("Use range");
+	$checkbox->show;
+	$hbox->pack_start ($checkbox, 0, 1, 0);
+	$checkbox->signal_connect (
+		"clicked", sub {
+			my $title = $self->selected_title;
+			return 1 if not $title;
+			return 1 if $self->in_transcode_init;
+			$title->set_tc_video_bitrate_range($_[0]->active);
+			$self->calc_video_bitrate;
+			$self->init_transcode_values;
+			1;
+		}
+	);
+	
+	$self->transcode_widgets->{tc_video_bitrate_range} = $checkbox;
+	$table->attach_defaults ($hbox, 2, 3, $row, $row+1);
+	
 	# Video Bitrate
 	++$row;
 	$hbox = Gtk::HBox->new;
@@ -560,17 +633,42 @@ sub create_video_bitrate_calc {
 		may_empty      => 0,
 	);
 	$entry->show;
-	$entry->set_usize(80,undef);
+	$entry->set_usize(65,undef);
 	$hbox->pack_start($entry, 0, 1, 0);
 
 	$label = Gtk::Label->new ("kbit/s");
 	$label->show;
 	$hbox->pack_start ($label, 0, 1, 0);
 	
+
 	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
 
 	$self->transcode_widgets->{tc_video_bitrate} = $entry;
 
+	$hbox = Gtk::HBox->new;
+	$hbox->show;	
+	my $checkbox = Gtk::CheckButton->new ("Manual");
+	$checkbox->show;
+	$hbox->pack_start ($checkbox, 0, 1, 0);
+	$checkbox->signal_connect (
+		"clicked", sub {
+			my $title = $self->selected_title;
+			return 1 if not $title;
+			return 1 if $self->in_transcode_init;
+			$title->set_tc_video_bitrate_manual($_[0]->active);
+			if ( not $title->tc_video_bitrate_manual ) {
+				$self->calc_video_bitrate;
+			}
+			$self->init_transcode_values;
+			1;
+		}
+	);
+	
+	$self->transcode_widgets->{tc_video_bitrate_manual} = $checkbox;
+
+	$table->attach_defaults ($hbox, 2, 3, $row, $row+1);
+	
+	
 	return $frame;
 }
 
@@ -680,9 +778,16 @@ sub create_general_options {
 	$entry = Video::DVDRip::CheckedEntry->new ( undef,
 		is_number => 1,
 		may_empty => 1,
+		cond => sub {
+			my $start = $self->transcode_widgets->{tc_start_frame}->get_text;
+			my $end   = $self->transcode_widgets->{tc_end_frame}->get_text;
+			return 1 if $start eq '' or $end eq '';
+			return $start < $end;
+		},
 	);
 	$entry->show;
 	$entry->set_usize (50,undef);
+	
 	$hbox->pack_start ($entry, 0, 1, 0);
 
 	$self->transcode_widgets->{tc_start_frame} = $entry;
@@ -694,9 +799,16 @@ sub create_general_options {
 	$entry = Video::DVDRip::CheckedEntry->new ( undef,
 		is_number => 1,
 		may_empty => 1,
+		cond => sub {
+			my $start = $self->transcode_widgets->{tc_start_frame}->get_text;
+			my $end   = $self->transcode_widgets->{tc_end_frame}->get_text;
+			return 1 if $start eq '' or $end eq '';
+			return $start < $end;
+		},
 	);
 	$entry->show;
 	$entry->set_usize (50,undef);
+	
 	$hbox->pack_start ($entry, 0, 1, 0);
 
 	$self->transcode_widgets->{tc_end_frame} = $entry;
@@ -815,7 +927,7 @@ sub create_general_options {
 
 	$self->transcode_widgets->{tc_execute_afterwards} = $entry;
 
-	my $checkbox = Gtk::CheckButton->new ("and exit");
+	my $checkbox = Gtk::CheckButton->new ("And exit");
 	$checkbox->show;
 	$hbox->pack_start ($checkbox, 1, 1, 0);
 	$self->transcode_widgets->{tc_exit_afterwards} = $checkbox;
@@ -829,7 +941,7 @@ sub create_calculated_storage {
 	my ($hsize_group) = @par{'hsize_group'};
 
 	my ($frame, $frame_hbox, $table, $row, $hbox, $label, $entry);
-	my ($popup_menu, $popup, $item, %popup_entries);
+	my ($popup_menu, $popup, $item, %popup_entries, $button);
 
 	# Frame
 	$frame = Gtk::Frame->new ("Calculated storage");
@@ -842,7 +954,7 @@ sub create_calculated_storage {
 	$frame->add ($frame_hbox);
 
 	# Table
-	$table = Gtk::Table->new ( 4, 2, 0 );
+	$table = Gtk::Table->new ( 4, 3, 0 );
 	$table->show;
 	$table->set_row_spacings ( $TABLE_SPACING );
 	$table->set_col_spacings ( $TABLE_SPACING );
@@ -852,7 +964,7 @@ sub create_calculated_storage {
 	$row = 0;
 	$hbox = Gtk::HBox->new;
 	$hbox->show;
-	$label = Gtk::Label->new ("Video size (MB):");
+	$label = Gtk::Label->new ("Video size");
 	$label->show;
 	$hbox->pack_start($label, 0, 1, 0);
 	$table->attach_defaults ($hbox, 0, 1, $row, $row+1);
@@ -862,16 +974,21 @@ sub create_calculated_storage {
 	$hbox->show;
 	$label = Gtk::Label->new ("100");
 	$label->show;
+	$label->set_justify("right");
 	$hbox->pack_start($label, 0, 1, 0);
 	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
 
 	$self->transcode_widgets->{label_size_video} = $label;
 
+	$label = Gtk::Label->new ("MB");
+	$label->show;
+	$hbox->pack_start($label, 0, 1, 0);
+
 	# Audio
 	++$row;
 	$hbox = Gtk::HBox->new;
 	$hbox->show;
-	$label = Gtk::Label->new ("Audio size (MB):");
+	$label = Gtk::Label->new ("Non video size");
 	$label->show;
 	$hbox->pack_start($label, 0, 1, 0);
 	$table->attach_defaults ($hbox, 0, 1, $row, $row+1);
@@ -881,11 +998,16 @@ sub create_calculated_storage {
 	$hbox->show;
 	$label = Gtk::Label->new ("100");
 	$label->show;
+	$label->set_justify("right");
 	$hbox->pack_start($label, 0, 1, 0);
 	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
 
 	$self->transcode_widgets->{label_size_audio} = $label;
 
+	$label = Gtk::Label->new ("MB");
+	$label->show;
+	$hbox->pack_start($label, 0, 1, 0);
+
 	# Separator
 	++$row;
 	my $sep = Gtk::HSeparator->new;
@@ -894,7 +1016,7 @@ sub create_calculated_storage {
 
 	# Sum
 	++$row;
-	$label = Gtk::Label->new ("Total size (MB):");
+	$label = Gtk::Label->new ("Total size");
 	$label->show;
 	$hbox = Gtk::HBox->new;
 	$hbox->show;
@@ -906,18 +1028,38 @@ sub create_calculated_storage {
 	$hbox->show;
 	$label = Gtk::Label->new ("100");
 	$label->show;
+	$label->set_justify("right");
 	$hbox->pack_start($label, 0, 1, 0);
+
 	$table->attach_defaults ($hbox, 1, 2, $row, $row+1);
 
 	$self->transcode_widgets->{label_size_total} = $label;
 
+	$label = Gtk::Label->new ("MB");
+	$label->show;
+	$hbox->pack_start($label, 0, 1, 0);
+
+	# details button
+	$hbox = Gtk::HBox->new;
+	$hbox->show;
+	$label = Gtk::Label->new ("   ");
+	$label->show;
+	$hbox->pack_start($label, 0, 1, 0);
+	$button = Gtk::Button->new (" Show details... ");
+	$button->show;
+	$hbox->pack_start($button, 0, 1, 0);
+
+	$table->attach_defaults ($hbox, 2, 3, $row, $row+1);
+
+	$button->signal_connect ("clicked", sub { $self->show_calc_details } );
+
 	return $frame;
 }
 
 sub init_transcode_values {
 	my $self = shift; $self->trace_in;
 	my %par = @_;
-	my ($no_audio) = @par{'no_audio'};
+	my ($no_audio, $no_codec_combo) = @par{'no_audio','no_codec_combo'};
 
 	my $title = $self->selected_title;
 	return 1 if not $title;
@@ -927,7 +1069,7 @@ sub init_transcode_values {
 
 	$self->set_in_transcode_init(1);
 
-	foreach my $attr (qw ( tc_video_codec tc_options tc_nice
+	foreach my $attr (qw ( tc_options tc_nice
 			       tc_video_af6_codec tc_video_bitrate
 			       tc_video_framerate tc_target_size
 			       tc_start_frame tc_end_frame
@@ -935,7 +1077,6 @@ sub init_transcode_values {
 		$widgets->{$attr}->set_text ($self->selected_title->$attr());
 	}
 
-	my $yuv             = $title->tc_use_yuv_internal;
 	my $chapter_mode    = $title->tc_use_chapter_mode;
 	my $multipass       = $title->tc_multipass;
 	my $deinterlace     = $title->tc_deinterlace;
@@ -944,19 +1085,19 @@ sub init_transcode_values {
 	my $video_codec     = $title->tc_video_codec;
 	my $psu_core        = $title->tc_psu_core;
 	my $exit_afterwards = $title->tc_exit_afterwards;
-	
+	my $container 	    = $title->tc_container;
+
 	my $disc_cnt        = $title->tc_disc_cnt;
 	my $disc_size       = $title->tc_disc_size;
 
-	$widgets->{tc_use_yuv_internal_yes}->set_active($yuv);
-	$widgets->{tc_use_yuv_internal_no}->set_active(!$yuv);
-
 	$widgets->{tc_multipass_yes}->set_active($multipass);
 	$widgets->{tc_multipass_no}->set_active(!$multipass);
 
 	$deinterlace = 4 if $deinterlace eq '32detect';
+	$deinterlace = 5 if $deinterlace eq 'smart';
+
 	$widgets->{tc_deinterlace_popup}->set_history ($deinterlace);
-	$widgets->{tc_anti_alias_popup}->set_history ($anti_alias);
+#	$widgets->{tc_anti_alias_popup}->set_history ($anti_alias);
 
 	$widgets->{tc_disc_cnt_popup}->set_history ($disc_cnt-1);
 	$widgets->{tc_disc_size_combo}->entry->set_text ($disc_size);
@@ -975,6 +1116,11 @@ sub init_transcode_values {
 		$self->switch_to_divx_encoding;
 	}
 
+	$container = $container eq 'avi' ? 0 :
+		     $container eq 'ogg' ? 1 : 2;
+
+	$widgets->{tc_container_popup}->set_history($container);
+
 	$self->set_in_transcode_init(0);
 
 	if ( not $no_audio ) {
@@ -982,6 +1128,24 @@ sub init_transcode_values {
 		$self->fill_target_audio_popup;
 	}
 
+	if ( not $no_codec_combo ) {
+		$self->init_video_codec_combo;
+	}
+
+	if ( $title->tc_video_bitrate_manual ) {
+		$widgets->{tc_video_bitrate_manual}->set_active(1);
+		$widgets->{tc_video_bitrate}->set_sensitive(1);
+	} else {
+		$widgets->{tc_video_bitrate_manual}->set_active(0);
+		$widgets->{tc_video_bitrate}->set_sensitive(0);
+	}
+
+	if ( $title->tc_video_bitrate_range ) {
+		$widgets->{tc_video_bitrate_range}->set_active(1);
+	} else {
+		$widgets->{tc_video_bitrate_range}->set_active(0);
+	}
+	
 	$self->update_storage_labels;
 
 	1;
@@ -1005,19 +1169,6 @@ sub switch_to_mpeg_encoding {
 	$widgets->{cluster_button}->set_sensitive(0);
 	$widgets->{view_avi_button}->child->set("View MPEG");
 
-	if ( $video_codec eq 'VCD' ) {
-		$widgets->{tc_video_bitrate}->set_sensitive(0);
-		$widgets->{tc_disc_cnt_popup}->set_sensitive(0);
-		$widgets->{tc_disc_size_combo}->set_sensitive(0);
-		$widgets->{tc_target_size}->set_text("");
-		$widgets->{tc_target_size}->set_sensitive(0);
-	} else {
-		$widgets->{tc_video_bitrate}->set_sensitive(1);
-		$widgets->{tc_disc_cnt_popup}->set_sensitive(1);
-		$widgets->{tc_disc_size_combo}->set_sensitive(1);
-		$widgets->{tc_target_size}->set_sensitive(1);
-	}
-
 	$_->hide foreach @{$widgets->{tc_mpeg_hide}};
 
 	1;
@@ -1035,14 +1186,10 @@ sub switch_to_divx_encoding {
 		$widgets->{$attr}->set_sensitive(1);
 	}
 
-	if ( $title->tc_audio_codec eq 'mp2' ) {
-		# just switching from SVCD to DivX
-		$title->set_tc_audio_codec ('mp3');
-	}
-
 	$widgets->{avisplit_button}->set_sensitive(1);
+	$widgets->{cluster_button}->set_sensitive(1);
 
-	if ( $title->tc_audio_codec eq 'ogg' ) {
+	if ( $title->is_ogg ) {
 		$widgets->{view_avi_button}->child->set("View OGG");
 		$widgets->{avisplit_button}->child->set("Split OGG");
 	} else {
@@ -1050,11 +1197,31 @@ sub switch_to_divx_encoding {
 		$widgets->{avisplit_button}->child->set("Split AVI");
 	}
 
-	$widgets->{tc_video_bitrate}->set_sensitive(1);
-	$widgets->{tc_disc_cnt_popup}->set_sensitive(1);
-	$widgets->{tc_disc_size_combo}->set_sensitive(1);
-	$widgets->{tc_target_size}->set_sensitive(1);
-	$widgets->{cluster_button}->set_sensitive(1);
+	1;
+}
+
+sub init_video_codec_combo {
+	my $self = shift;
+
+	my $title = $self->selected_title;
+	return 1 if not $title;
+
+	my $in_init = $self->in_transcode_init;
+	$self->set_in_transcode_init(1);
+
+	my $widgets = $self->transcode_widgets;
+
+	if ( $title->tc_container eq 'vcd' ) {
+		$widgets->{tc_video_codec_combo}
+			->set_popdown_strings( "SVCD", "VCD" );
+	} else {
+		$widgets->{tc_video_codec_combo}
+			->set_popdown_strings( "divx4","divx5","xvid","xvidcvs","ffmpeg","fame","af6" );
+	}
+
+	$widgets->{tc_video_codec}->set_text($title->tc_video_codec);
+
+	$self->set_in_transcode_init($in_init);
 
 	1;
 }
@@ -1066,7 +1233,6 @@ sub calc_video_bitrate {
 	return 1 if not $title;
 	
 	$title->calc_video_bitrate;
-
 	$self->transcode_widgets->{tc_video_bitrate}->set_text($title->tc_video_bitrate);
 	
 	1;
@@ -1078,35 +1244,21 @@ sub update_storage_labels {
 	my $title = $self->selected_title;
 	return 1 if not $title;
 
+	my $bc = Video::DVDRip::BitrateCalc->new (
+		title => $title,
+	);
+	$bc->calculate_video_bitrate;
+
 	my $video_label = $self->transcode_widgets->{label_size_video};
 	my $audio_label = $self->transcode_widgets->{label_size_audio};
 	my $total_label = $self->transcode_widgets->{label_size_total};
 
-	my $frames        = $title->frames;
-	my $fps           = $title->frame_rate;
-	my $video_bitrate = $title->tc_video_bitrate;
+	$video_label->set_text(sprintf("%.2f",$bc->video_size));
+	$audio_label->set_text(sprintf("%.2f",$bc->non_video_size));
+	$total_label->set_text(sprintf("%.2f",$bc->video_size + $bc->non_video_size));
 
-	my $runtime = $frames/$fps;
-	my $video_size = int($runtime * $video_bitrate * 1000 / 1024 / 1024 / 8);
-
-	my $audio_size_text = "";
-	my $audio_sum = 0;
-
-	foreach my $audio ( @{$title->tc_audio_tracks} ) {
-		next if $audio->tc_target_track == -1;
-		my $size = int($runtime * $audio->tc_bitrate / 1000 / 8);
-		$audio_size_text .= "$size + ";
-		$audio_sum       += $size;
-	}
-
-	$audio_size_text =~ s/ \+ $//;
-	$audio_size_text = "$audio_sum    = $audio_size_text";
-
-	my $total = $video_size + $audio_sum;
-
-	$video_label->set_text($video_size);
-	$audio_label->set_text($audio_size_text);
-	$total_label->set_text($total);
+	my $bc_window = eval { $self->comp('bitrate_calc') };
+	$bc_window->init_calc_list if $bc_window;
 
 	1;
 }
@@ -1179,14 +1331,43 @@ sub transcode {
 		);
 	}
 
-	if ( $mpeg and keys %{$title->get_additional_audio_tracks} ) {
+	if ( keys %{$title->get_additional_audio_tracks} ) {
+		if ( $title->tc_video_codec eq 'VCD' ) {
+			$self->message_window (
+				message =>
+					"Having more than one audio track ".
+					"isn't possible on a VCD."
+			);
+			return 1;
+		}
+		if ( $title->tc_video_codec eq 'SVCD' and
+		     keys %{$title->get_additional_audio_tracks} > 1 ) {
+			$self->message_window (
+				message =>
+					"WARNING: Having more than two audio tracks\n".
+					"on a SVCD is not standard conform. You may\n".
+					"encounter problems on hardware players."
+			);
+		}
+	}
+
+	my $svcd_warning;
+	if ( $svcd_warning = $title->check_svcd_geometry ) {
 		$self->message_window (
-			message => "WARNING: additional audio tracks for (S)VCD\n".
-				   "currently not supported. Only first track\n".
-				   "will be transcoded.",
+			message =>
+				"WARNING: $svcd_warning\n".
+				"You better cancel now and select the appropriate\n".
+				"preset on the Clip & Zoom page.",
 		);
 	}
 
+	return $self->transcode_multipass_with_vbr_audio (
+		split         => $split,
+		subtitle_test => $subtitle_test
+	) if not $subtitle_test
+	     and $title->has_vbr_audio
+	     and $title->tc_multipass;
+
 	my $nr;
 	my $job;
 	my $last_job;
@@ -1199,7 +1380,8 @@ sub transcode {
 		);
 		$job->set_chapter ($chapter);
 		$job->set_subtitle_test ($subtitle_test);
-	
+		$job->set_split ($split);
+
 		if ( not $subtitle_test and $title->tc_multipass ) {
 			$job->set_pass (1);
 			$last_job = $exec->add_job ( job => $job );
@@ -1209,6 +1391,7 @@ sub transcode {
 				title         => $title,
 			);
 			$job->set_pass (2);
+			$job->set_split ($split);
 			$job->set_chapter ($chapter);
 			$job->set_depends_on_jobs ( [ $last_job ] );
 			$last_job = $exec->add_job ( job => $job );
@@ -1218,7 +1401,7 @@ sub transcode {
 			$last_job = $exec->add_job ( job => $job );
 		}
 
-		if ( $title->tc_audio_codec eq 'ogg' ) {
+		if ( $title->tc_container eq 'ogg' ) {
 			$job = Video::DVDRip::Job::MergeAudio->new (
 				nr    => ++$nr,
 				title => $title,
@@ -1230,7 +1413,7 @@ sub transcode {
 			$last_job = $exec->add_job ( job => $job );
 		}
 
-		if ( not $mpeg and not $subtitle_test ) {
+		if ( not $subtitle_test ) {
 			my $add_audio_tracks = $title->get_additional_audio_tracks;
 			if ( keys %{$add_audio_tracks} ) {
 				my ($avi_nr, $vob_nr);
@@ -1246,14 +1429,16 @@ sub transcode {
 					$job->set_chapter ($chapter);
 					$last_job = $exec->add_job ( job => $job );
 
-					$job = Video::DVDRip::Job::MergeAudio->new (
-						nr    => ++$nr,
-						title => $title,
-					);
-					$job->set_vob_nr ( $vob_nr );
-					$job->set_avi_nr ( $avi_nr );
-					$job->set_chapter ($chapter);
-					$last_job = $exec->add_job ( job => $job );
+					if ( not $mpeg ) {
+						$job = Video::DVDRip::Job::MergeAudio->new (
+							nr    => ++$nr,
+							title => $title,
+						);
+						$job->set_vob_nr ( $vob_nr );
+						$job->set_avi_nr ( $avi_nr );
+						$job->set_chapter ($chapter);
+						$last_job = $exec->add_job ( job => $job );
+					}
 				}
 			}
 		}
@@ -1318,6 +1503,160 @@ sub transcode {
 	1;
 }
 
+sub transcode_multipass_with_vbr_audio {
+	my $self = shift;
+	my %par = @_;
+	my ($split) = @par{'split'};
+
+	$self->log ("This title is transcoded with vbr audio ".
+		    "and video bitrate optimization.");
+
+	my $title    = $self->selected_title;
+	my $chapters = $title->get_chapters;
+
+	if ( not $title->tc_use_chapter_mode ) {
+		$chapters = [ undef ];
+	}
+
+	my $bc = Video::DVDRip::BitrateCalc->new (
+		title => $title,
+	);
+
+	my $nr;
+	my $job;
+	my $last_job;
+	my $exec = Video::DVDRip::GUI::ExecuteJobs->new;
+
+	# 1. encode additional audio tracks
+	foreach my $chapter ( @{$chapters} ) {
+		my $add_audio_tracks = $title->get_additional_audio_tracks;
+		if ( keys %{$add_audio_tracks} ) {
+			my ($avi_nr, $vob_nr);
+			foreach $avi_nr ( sort keys %{$add_audio_tracks} ) {
+				$vob_nr = $add_audio_tracks->{$avi_nr};
+				$job = Video::DVDRip::Job::TranscodeAudio->new (
+					nr    => ++$nr,
+					title => $title,
+				);
+				$job->set_vob_nr  ( $vob_nr  );
+				$job->set_avi_nr  ( $avi_nr  );
+				$job->set_chapter ( $chapter );
+				$job->set_bc      ( $bc      );
+				$last_job = $exec->add_job ( job => $job );
+			}
+		}
+	}
+	
+	# 2. 1st pass of Video + 1st Audio track
+	foreach my $chapter ( @{$chapters} ) {
+		$job  = Video::DVDRip::Job::TranscodeVideo->new (
+			nr    => ++$nr,
+			title => $title,
+		);
+		$job->set_bc ($bc);
+		$job->set_pass (1);
+		$job->set_chapter ($chapter);
+		$job->set_depends_on_jobs ( [ $last_job ] ) if $last_job;
+		$last_job = $exec->add_job ( job => $job );
+	}
+	
+	# 3. after 1st pass: calculate video bitrate (real audio size known)
+	$last_job->set_cb_finished (sub {
+		$title->set_tc_video_bitrate ( $bc->calculate_video_bitrate );
+		$self->log ("Adjusted video bitrate to ".
+			    $title->tc_video_bitrate.
+			    " after vbr audio transcoding");
+		$self->init_transcode_values;
+		1;
+	});
+	
+	# 4. 2nd pass Video and merging
+	foreach my $chapter ( @{$chapters} ) {
+		# transcode video 2nd pass
+		$job = Video::DVDRip::Job::TranscodeVideo->new (
+			nr            => ++$nr,
+			title         => $title,
+		);
+		$job->set_pass (2);
+		$job->set_chapter ($chapter);
+		$job->set_depends_on_jobs ( [ $last_job ] );
+		$last_job = $exec->add_job ( job => $job );
+
+		# merge 1st audio track
+		$job = Video::DVDRip::Job::MergeAudio->new (
+			nr    => ++$nr,
+			title => $title,
+		);
+		$job->set_vob_nr ( $title->get_first_audio_track );
+		$job->set_avi_nr ( 0 );
+		$job->set_chapter ($chapter);
+		$last_job = $exec->add_job ( job => $job );
+
+		# merge add. audio tracks
+		my ($avi_nr, $vob_nr);
+		my $add_audio_tracks = $title->get_additional_audio_tracks;
+		foreach $avi_nr ( sort keys %{$add_audio_tracks} ) {
+			$vob_nr = $add_audio_tracks->{$avi_nr};
+
+			$job = Video::DVDRip::Job::MergeAudio->new (
+				nr    => ++$nr,
+				title => $title,
+			);
+			$job->set_vob_nr ( $vob_nr );
+			$job->set_avi_nr ( $avi_nr );
+			$job->set_chapter ($chapter);
+			$last_job = $exec->add_job ( job => $job );
+		}
+
+	}
+
+	# 5. optional splitting (non chapter mode only)
+	if ( $split ) {
+		$job = Video::DVDRip::Job::Split->new (
+			nr    => ++$nr,
+			title => $title,
+		);
+		$job->set_depends_on_jobs ( [ $last_job ] );
+		$last_job = $exec->add_job ( job => $job );
+
+		$last_job->set_cb_finished ( sub {
+			$self->create_splitted_vobsub (
+				exec     => $exec,
+				last_job => $last_job
+			);
+			1;
+		} );
+	} else {
+		$last_job->set_cb_finished ( sub {
+			$self->create_non_splitted_vobsub (
+				exec     => $exec,
+				last_job => $last_job
+			);
+			1;
+		} );
+	}
+
+	# 6. execute afterwards stuff
+	$exec->set_cb_finished (sub {
+		return 1 if $exec->cancelled or $exec->errors_occured;
+		if ( $title->tc_execute_afterwards =~ /\S/ ) {
+			system ("(".$title->tc_execute_afterwards.") &");
+		}
+		if ( $title->tc_exit_afterwards ) {
+			$title->project->save
+				if $title->tc_exit_afterwards ne 'dont_save';
+			$self->comp('main')->exit_program (
+				force => ($title->tc_exit_afterwards eq 'dont_save')
+			);
+		}
+		1;
+	});
+
+	$exec->execute_jobs;
+
+	1;
+}
+
 sub avisplit {
 	my $self = shift;
 
@@ -1385,12 +1724,6 @@ sub scan_rescale_volume {
 	return 1 if $self->comp('progress')->is_active;
 	return 1 if not $title;
 
-	if ( $title->tc_use_chapter_mode ) {
-		$self->message_window (
-			message => "Chapter mode not yet supported"
-		);
-	}
-
 	if ( not $title->is_ripped ) {
 		$self->message_window (
 			message => "You first have to rip this title."
@@ -1398,17 +1731,29 @@ sub scan_rescale_volume {
 		return 1;
 	}
 
+	my $job;
 	my $nr;
+        my $chapter_count = 0;
 	my $last_job;
 	my $exec = Video::DVDRip::GUI::ExecuteJobs->new (
 		cb_finished => sub { $self->init_transcode_values }
 	);
-	my $job  = Video::DVDRip::Job::ScanVolume->new (
-		nr    => ++$nr,
-		title => $title,
-	);
 
-	$last_job = $exec->add_job ( job => $job );
+	my $chapters = $title->get_chapters;
+
+	if ( not $title->tc_use_chapter_mode ) {
+		$chapters = [ undef ];
+	}
+
+	foreach my $chapter ( @{$chapters} ) {
+		$job  = Video::DVDRip::Job::ScanVolume->new (
+			nr    => ++$nr,
+			title => $title,
+		);
+		$job->set_chapter ($chapter);
+		$job->set_count   ($chapter_count++);
+		$last_job = $exec->add_job ( job => $job );
+	}
 
 	$exec->execute_jobs;
 
@@ -1457,6 +1802,13 @@ sub add_to_cluster {
 		return 1;
 	}
 
+	if ( $title->tc_container eq 'vcd' ) {
+		$self->message_window (
+			message => "(S)VCD is not supported for cluster mode."
+		);
+		return 1;
+	}
+
 	# calculate program stream units, if not already done
 	$title->calc_program_stream_units
 		if not $title->program_stream_units or
@@ -1488,4 +1840,63 @@ sub add_to_cluster {
 	1;
 }
 
+sub show_calc_details {
+	my $self = shift;
+
+	my $title = $self->selected_title;
+	return 1 if not $title;
+
+	require Video::DVDRip::GUI::BitrateCalc;
+
+	my $calc_details = Video::DVDRip::GUI::BitrateCalc->new;
+	$calc_details->open_window;
+
+	1;	
+}
+
+sub create_wav {
+	my $self = shift;
+
+	my $title = $self->selected_title;
+	return 1 if not $title;
+
+	my $nr;
+	my $last_job;
+	my $exec = Video::DVDRip::GUI::ExecuteJobs->new;
+
+	my $chapters = $title->get_chapters;
+
+	if ( not $title->tc_use_chapter_mode ) {
+		$chapters = [ undef ];
+	}
+
+	my $job;
+	foreach my $chapter ( @{$chapters} ) {
+		$job  = Video::DVDRip::Job::CreateWav->new (
+			nr    => ++$nr,
+			title => $title,
+		);
+		$job->set_chapter ( $chapter );
+		$last_job = $exec->add_job ( job => $job );
+	}
+
+	$exec->execute_jobs;
+
+	1;
+}
+
+sub transcode_filters {
+	my $self = shift;
+
+	my $title = $self->selected_title;
+	return 1 if not $title;
+
+	require Video::DVDRip::GUI::Filters;
+
+	my $filters = Video::DVDRip::GUI::Filters->new;
+	$filters->open_window;
+
+	1;	
+}
+
 1;
